@@ -27,9 +27,11 @@ format created here.
 __author__ = ('jaq@google.com (Jamie Wilkinson)',
               'vasilios@google.com (Vasilios Hoffman)')
 
+import errno
 import os.path
 import re
 import shutil
+import stat
 import sys
 
 from nss_cache import config
@@ -231,7 +233,7 @@ class FilesCache(caches.Cache):
         shutil.copymode(self.GetCompatFilename(), index_filename)
       except OSError, e:
         if e.errno == errno.ENOENT:
-          os.chmod(self.index_filename,
+          os.chmod(index_filename,
                    stat.S_IRUSR|stat.S_IWUSR|stat.S_IRGRP|stat.S_IROTH)
       for key in sorted(index):
         pos = index[key]
