@@ -1,4 +1,4 @@
-#!/usr/bin/python2.4
+#!/usr/bin/python
 #
 # Copyright 2007 Google Inc.
 #
@@ -21,23 +21,26 @@
 __author__ = 'jaq@google.com (Jamie Wilkinson)'
 
 import bsddb
+import fcntl
 import os
-import popen2
+import select
+import subprocess
 
 from nss_cache import config
 from nss_cache import error
-from nss_cache import maps
-from nss_cache.caches import base
+from nss_cache.caches import caches
+from nss_cache.maps import group 
+from nss_cache.maps import passwd
+from nss_cache.maps import shadow
 
-# python2.3 has no builtin set() class
-try:
-  Set = set
-except NameError:
-  import sets
-  Set = sets.Set
+def RegisterAllImplementations(register_callback):
+  """Register our cache classes independently from the import scheme."""
+  register_callback('nssdb', 'passwd', NssDbPasswdHandler)
+  register_callback('nssdb', 'group', NssDbGroupHandler)
+  register_callback('nssdb', 'shadow', NssDbShadowHandler)
 
 
-class NssDbCache(base.Cache):
+class NssDbCache(caches.Cache):
   """An implementation of a Cache specific to nss_db.
 
   nss_db uses one Berkeley DB database per map for the cache.  This class
@@ -50,26 +53,26 @@ class NssDbCache(base.Cache):
   UPDATE_TIMESTAMP_SUFFIX = 'nsscache-update-timestamp'
   MODIFY_TIMESTAMP_SUFFIX = 'nsscache-timestamp'
 
-  def __init__(self, conf, map_name, automount_info=None):
+  def __init__(self, conf, map_name, automount_mountpoint=None):
     """Create a handler for the given map type.
 
     Args:
      conf: a configuration object
      map_name: a string representing the type of map we are
-     automount_info: A string containing the automount mountpoint, used only
-       by automount maps.
+     automount_mountpoint: A string containing the automount mountpoint,
+       used only by automount maps.
 
     Returns: A CacheMapHandler instance.
     """
     super(NssDbCache, self).__init__(conf, map_name,
-                                     automount_info=automount_info)
+                                     automount_mountpoint=automount_mountpoint)
     self.makedb = conf.get('makedb', '/usr/bin/makedb')
 
-  def GetMap(self, cache_info=None):
+  def GetMap(self, cache_filename=None):
     """Returns the map from the cache.
 
     Args:
-      cache_info: unused by this implementation of base.Cache
+      cache_filename: unused by this implementation of caches.Cache
     Returns:
       a Map containing the map cache
     """
@@ -107,7 +110,7 @@ class NssDbCache(base.Cache):
     """Run 'makedb' in a subprocess and return it to use for streaming.
 
     Returns:
-      (tochild, fromchild): stdin and stdout of makedb subprocess
+      a subprocess object
     """
     # TODO(jaq): this should probably raise a better exception and be handled
     # gracefully
@@ -115,8 +118,27 @@ class NssDbCache(base.Cache):
       self.log.warn('makedb binary %s does not exist, cannot generate '
                     'bdb map', self.makedb)
     self.log.debug('executing makedb: %s - %s',
-                   self.makedb, self.cache_filename)
-    return popen2.Popen4([self.makedb, '-', self.cache_filename])
+                   self.makedb, self.temp_cache_filename)
+    # This is a race condition on the tempfile now, but db-4.8 is braindead
+    # and refuses to open zero length files with:
+    # fop_read_meta: foo: unexpected file type or format
+    # foo: Invalid type 5 specified
+    # makedb: cannot open output file `foo': Invalid argument
+    os.unlink(self.temp_cache_filename)
+    makedb = subprocess.Popen([self.makedb, '-', self.temp_cache_filename],
+                              stdin=subprocess.PIPE,
+                              stdout=subprocess.PIPE,
+                              stderr=subprocess.STDOUT,
+                              close_fds=True)
+    fcntl.fcntl(makedb.stdout, fcntl.F_SETFL, os.O_NONBLOCK)
+    makedb.allout = ""
+    return makedb
+
+  def _Read(self, proc):
+    while len(select.select([proc.stdout],(),(),0)[0]) > 0:
+      data = proc.stdout.read()
+      if len(data) == 0: break # sigh... select() says there's data.
+      proc.allout += data
 
   def Write(self, map_data):
     """Write the map to the cache file.
@@ -129,36 +151,43 @@ class NssDbCache(base.Cache):
       map_data: A Map subclass
 
     Returns:
-      a Set() of keys written or None on failure.
+      a set of keys written or None on failure.
     """
     self._Begin()
-    written_keys = Set()
+    written_keys = set()
 
+    self.log.debug('Map contains %d elems', len(map_data))
+
+    enumeration_index = 0
+    makedb = self._SpawnMakeDb()
+    self.makedbproc = makedb
     try:
-      makedb = self._SpawnMakeDb()
-      enumeration_index = 0
 
       try:
-        while 1:
+        while True:
           entry = map_data.PopItem()
-          if makedb.poll() != -1:
+          self._Read(makedb)
+          if makedb.poll() is not None:
             self.log.error('early exit from makedb! child output: %s',
-                           makedb.fromchild.read())
+                           makedb.allout)
             # in this case, no matter how the child exited, we complain
             return None
-          self.WriteData(makedb.tochild, entry, enumeration_index)
+          self.WriteData(makedb.stdin, entry, enumeration_index)
           written_keys.update(self.ExpectedKeysForEntry(entry))
           enumeration_index += 1
       except KeyError:
         # expected when PopItem() is done, and breaks our loop for us.
         pass
 
-      makedb.tochild.close()
+      makedb.stdin.close()
       self.log.debug('%d entries written, %d keys', enumeration_index,
                      len(written_keys))
 
-      map_data = makedb.fromchild.read()
-      makedb.fromchild.close()
+      # wait for subprocess to commit data before we move live.
+      makedb.wait()
+      self._Read(makedb)
+      makedb.stdout.close()
+      map_data = makedb.allout
       if map_data:
         self.log.debug('makedb output: %r', map_data)
 
@@ -168,6 +197,10 @@ class NssDbCache(base.Cache):
       return None
 
     except:
+      self.log.debug('Wrote %d entries before exception', enumeration_index)
+      self.log.debug('makedb output: %s', makedb.allout)
+      # wait for subprocess to commit data before we roll back.
+      makedb.wait()
       self._Rollback()
       raise
 
@@ -175,6 +208,8 @@ class NssDbCache(base.Cache):
     """Helper function to compute if a child exited with code 0 or not."""
     return os.WIFEXITED(code) and (os.WEXITSTATUS(code) is 0)
 
+  # TODO(jaq): validate the unit tests for this code path, are we
+  # verifying the temp cache or the real cache?
   def Verify(self, written_keys):
     """Verify that the written cache is correct.
 
@@ -182,19 +217,19 @@ class NssDbCache(base.Cache):
     back and verifying that it loads and has the entries we expect.
 
     Args:
-      written_keys: a Set() of keys that should have been written to disk.
+      written_keys: a set of keys that should have been written to disk.
 
     Returns:
       boolean indicating success.
 
     Raises:
-      EmptyMap: The map to be verified against is empty.
+      EmptyMap: The cache being verified is empty.
     """
     self.log.debug('verification started')
-    db = bsddb.btopen(self.cache_filename, 'r')
+    db = bsddb.btopen(self.temp_cache_filename, 'r')
     # cast keys to a set for fast __contains__ lookup in the loop
     # following
-    cache_keys = Set(db)
+    cache_keys = set(db)
     db.close()
 
     written_key_count = len(written_keys)
@@ -203,8 +238,8 @@ class NssDbCache(base.Cache):
                    cache_key_count)
 
     if cache_key_count <= 0 and written_key_count > 0:
-      # We have an empty map yet we should have written more.
-      # Uncaught disk full or other error?
+      # We have an empty db, yet we expect that earlier we should have
+      # written more. Uncaught disk full or other error?
       raise error.EmptyMap
 
     # makedb creates new keys internally.  we only care that all the keys
@@ -219,7 +254,7 @@ class NssDbCache(base.Cache):
       self._Rollback()
       return False
 
-    self.log.info('verify passed: %s', self.CACHE_FILENAME)
+    self.log.info('verify passed: %s', self.temp_cache_filename)
     return True
 
 
@@ -227,10 +262,10 @@ class NssDbPasswdHandler(NssDbCache):
   """Concrete class for updating a nss_db passwd cache."""
   CACHE_FILENAME = 'passwd.db'
 
-  def __init__(self, conf, map_name=None, automount_info=None):
+  def __init__(self, conf, map_name=None, automount_mountpoint=None):
     if map_name is None: map_name = config.MAP_PASSWORD
-    super(NssDbPasswdHandler, self).__init__(conf, map_name,
-                                             automount_info=automount_info)
+    super(NssDbPasswdHandler, self).__init__(
+        conf, map_name, automount_mountpoint=automount_mountpoint)
 
   def WriteData(self, target, entry, enumeration_index):
     """Generate three entries as expected by nss_db passwd map.
@@ -287,7 +322,7 @@ class NssDbPasswdHandler(NssDbCache):
       entry = entry[:-1]
 
     entry = entry.split(':')
-    map_entry = maps.PasswdMapEntry()
+    map_entry = passwd.PasswdMapEntry()
     # maps expect strict typing, so convert to int as appropriate.
     map_entry.name = entry[0]
     map_entry.passwd = entry[1]
@@ -312,17 +347,14 @@ class NssDbPasswdHandler(NssDbCache):
             '=%d' % entry.uid]
 
 
-base.RegisterImplementation('nssdb', 'passwd', NssDbPasswdHandler)
-
-
 class NssDbGroupHandler(NssDbCache):
   """Concrete class for updating nss_db group maps."""
   CACHE_FILENAME = 'group.db'
 
-  def __init__(self, conf, map_name=None, automount_info=None):
+  def __init__(self, conf, map_name=None, automount_mountpoint=None):
     if map_name is None: map_name = config.MAP_GROUP
-    super(NssDbGroupHandler, self).__init__(conf, map_name,
-                                            automount_info=automount_info)
+    super(NssDbGroupHandler, self).__init__(
+        conf, map_name, automount_mountpoint=automount_mountpoint)
 
   def WriteData(self, target, entry, enumeration_index):
     """Generate three entries as expected by nss_db group map.
@@ -374,7 +406,7 @@ class NssDbGroupHandler(NssDbCache):
       entry = entry[:-1]
 
     entry = entry.split(':')
-    map_entry = maps.GroupMapEntry()
+    map_entry = group.GroupMapEntry()
     # map entries expect strict typing, so convert as appropriate
     map_entry.name = entry[0]
     map_entry.passwd = entry[1]
@@ -396,17 +428,14 @@ class NssDbGroupHandler(NssDbCache):
             '=%d' % entry.gid]
 
 
-base.RegisterImplementation('nssdb', 'group', NssDbGroupHandler)
-
-
 class NssDbShadowHandler(NssDbCache):
   """Concrete class for updating nss_db shadow maps."""
   CACHE_FILENAME = 'shadow.db'
 
-  def __init__(self, conf, map_name=None, automount_info=None):
+  def __init__(self, conf, map_name=None, automount_mountpoint=None):
     if map_name is None: map_name = config.MAP_SHADOW
-    super(NssDbShadowHandler, self).__init__(conf, map_name,
-                                             automount_info=automount_info)
+    super(NssDbShadowHandler, self).__init__(
+        conf, map_name, automount_mountpoint=automount_mountpoint)
 
   def WriteData(self, target, entry, enumeration_index):
     """Generate three entries as expected by nss_db shadow map.
@@ -466,7 +495,7 @@ class NssDbShadowHandler(NssDbCache):
       entry = entry[:-1]
 
     entry = entry.split(':')
-    map_entry = maps.ShadowMapEntry()
+    map_entry = shadow.ShadowMapEntry()
     # map entries expect strict typing, so convert as appropriate
     map_entry.name = entry[0]
     map_entry.passwd = entry[1]
@@ -496,6 +525,3 @@ class NssDbShadowHandler(NssDbCache):
       a list of strings
     """
     return ['.%s' % entry.name]
-
-
-base.RegisterImplementation('nssdb', 'shadow', NssDbShadowHandler)
