@@ -23,11 +23,12 @@ __author__ = 'jaq@google.com (Jamie Wilkinson)'
 import bsddb
 import logging
 import os.path
+import select
 import shutil
+import sys
 import tempfile
 import time
 import unittest
-import sys
 
 import mox
 
@@ -43,6 +44,16 @@ class TestSkipped(Exception):
   """Exception to raise if a test cannot be run."""
 
 
+class MakeDbDummy(object):
+  allout = ""
+      
+  def wait(self):
+    return 0
+
+  def poll(self):
+    return None
+
+
 class TestNssDbPasswdHandler(mox.MoxTestBase):
 
   def setUp(self):
@@ -126,12 +137,9 @@ class TestNssDbPasswdHandler(mox.MoxTestBase):
     pw.Verify()
     self.failUnless(m.Add(pw))
 
-    class MakeDbDummy(object):
-      def wait(self):
-        return 0
-
-      def poll(self):
-        return None
+    self.mox.StubOutWithMock(select, 'select')
+    select.select([makedb_stdout], (), (), 0).AndReturn(([37], [], []))
+    select.select([makedb_stdout], (), (), 0).AndReturn(([], [], []))
 
     def SpawnMakeDb():
       makedb = MakeDbDummy()
@@ -309,13 +317,10 @@ class TestNssDbGroupHandler(mox.MoxTestBase):
     g.Verify()
     self.failUnless(m.Add(g))
 
-    class MakeDbDummy(object):
-      def wait(self):
-        return 0
-
-      def poll(self):
-        return None
-
+    self.mox.StubOutWithMock(select, 'select')
+    select.select([makedb_stdout], (), (), 0).AndReturn(([37], [], []))
+    select.select([makedb_stdout], (), (), 0).AndReturn(([], [], []))
+    
     def SpawnMakeDb():
       makedb = MakeDbDummy()
       makedb.stdin = makedb_stdin
@@ -465,12 +470,9 @@ class TestNssDbShadowHandler(mox.MoxTestBase):
     s.Verify()
     self.failUnless(m.Add(s))
 
-    class MakeDbDummy(object):
-      def wait(self):
-        return 0
-
-      def poll(self):
-        return None
+    self.mox.StubOutWithMock(select, 'select')
+    select.select([makedb_stdout], (), (), 0).AndReturn(([37], [], []))
+    select.select([makedb_stdout], (), (), 0).AndReturn(([], [], []))
 
     def SpawnMakeDb():
       makedb = MakeDbDummy()
