@@ -1,4 +1,4 @@
-#!/usr/bin/python2.4
+#!/usr/bin/python
 #
 # Copyright 2007 Google Inc.
 #
@@ -21,18 +21,24 @@
 __author__ = ('jaq@google.com (Jamie Wilkinson)',
               'vasilios@google.com (Vasilios Hoffman)')
 
-import calendar
 import logging
 import time
 import ldap
+import ldap.sasl
 
-from nss_cache import config
 from nss_cache import error
-from nss_cache import maps
-from nss_cache.sources import base
+from nss_cache.maps import automount
+from nss_cache.maps import group
+from nss_cache.maps import netgroup
+from nss_cache.maps import passwd
+from nss_cache.maps import shadow
+from nss_cache.sources import source
 
-
-class LdapSource(base.Source):
+def RegisterImplementation(registration_callback):
+  registration_callback(LdapSource)
+  
+  
+class LdapSource(source.Source):
   """Source for data in LDAP.
 
   After initialisation, one can search the data source for 'objects'
@@ -66,8 +72,9 @@ class LdapSource(base.Source):
     """
     super(LdapSource, self).__init__(conf)
     self._dn_requested = False  # dn is a special-cased attribute
-    
+
     self._SetDefaults(conf)
+    self._conf = conf
 
     if conn is None:
       # ReconnectLDAPObject should handle interrupted ldap transactions.
@@ -76,6 +83,8 @@ class LdapSource(base.Source):
       self.conn = rlo(uri=conf['uri'],
                       retry_max=conf['retry_max'],
                       retry_delay=conf['retry_delay'])
+      if conf['tls_starttls'] == 1:
+          self.conn.start_tls_s()
     else:
       self.conn = conn
 
@@ -107,6 +116,8 @@ class LdapSource(base.Source):
       configuration['tls_cacertdir'] = self.TLS_CACERTDIR
     if not 'tls_cacertfile' in configuration:
       configuration['tls_cacertfile'] = self.TLS_CACERTFILE
+    if not 'tls_starttls' in configuration:
+      configuration['tls_starttls'] = 0
 
     # Translate tls_require into appropriate constant, if necessary.
     if configuration['tls_require_cert'] == 'never':
@@ -119,6 +130,16 @@ class LdapSource(base.Source):
       configuration['tls_require_cert'] = ldap.OPT_X_TLS_ALLOW
     elif configuration['tls_require_cert'] == 'try':
       configuration['tls_require_cert'] = ldap.OPT_X_TLS_TRY
+    
+    if not 'sasl_authzid' in configuration:
+      configuration['sasl_authzid'] = ''
+
+    # Should we issue STARTTLS?
+    if configuration['tls_starttls'] in (1, '1', 'on', 'yes', 'true'):
+        configuration['tls_starttls'] = 1
+    #if not configuration['tls_starttls']:
+    else:
+      configuration['tls_starttls'] = 0
 
     # Setting global ldap defaults.
     ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT,
@@ -136,8 +157,19 @@ class LdapSource(base.Source):
       self.log.debug('opening ldap connection and binding to %s',
                      configuration['uri'])
       try:
-        self.conn.simple_bind_s(who=configuration['bind_dn'],
-                                cred=configuration['bind_password'])
+        if 'use_sasl' in configuration and configuration['use_sasl']:
+          if ('sasl_mech' in configuration and
+	      configuration['sasl_mech'] and
+              configuration['sasl_mech'].lower() == 'gssapi'):
+            sasl = ldap.sasl.gssapi(configuration['sasl_authzid'])
+          # TODO: Add other sasl mechs
+          else:
+            raise error.ConfigurationError('SASL mechanism not supported')
+
+          self.conn.sasl_interactive_bind_s('', sasl)
+        else:
+          self.conn.simple_bind_s(who=configuration['bind_dn'],
+                                cred=str(configuration['bind_password']))
         break
       except ldap.SERVER_DOWN, e:
         retry_count += 1
@@ -180,8 +212,25 @@ class LdapSource(base.Source):
       Search results from the prior call to self.Search()
     """
     while True:
-      result_type, data = self.conn.result(self.message_id, all=0,
-                                           timeout=self.conf['timelimit'])
+      result_type, data = None, None
+
+      timeout_retries = 0
+      while timeout_retries < self._conf['retry_max']:
+        try:
+          result_type, data = self.conn.result(self.message_id, all=0,
+                                               timeout=self.conf['timelimit'])
+	  break
+        except ldap.NO_SUCH_OBJECT:
+          self.log.debug('Returning due to ldap.NO_SUCH_OBJECT')
+          return
+        except ldap.TIMELIMIT_EXCEEDED:
+          timeout_retries += 1
+          self.log.warning('Timeout on LDAP results, attempt #%s.', timeout_retries)
+          if timeout_retries >= self._conf['retry_max']:
+            self.log.debug('max retries hit, returning')
+            return
+          self.log.debug('sleeping %d seconds', self._conf['retry_delay'])
+          time.sleep(self.conf['retry_delay'])
 
       if result_type == ldap.RES_SEARCH_RESULT:
         self.log.debug('Returning due to RES_SEARCH_RESULT')
@@ -198,7 +247,7 @@ class LdapSource(base.Source):
         # If the dn is requested, return it along with the payload,
         # otherwise ignore it.
         if self._dn_requested:
-          merged_records = {'dn':record[0]}
+          merged_records = {'dn': record[0]}
           merged_records.update(record[1])
           yield merged_records
         else:
@@ -276,11 +325,11 @@ class LdapSource(base.Source):
     set our search scope to be 'one'.
 
     Args:
-      location: Currently a string containing our search base, later we
-        may support hostname and additional parameters.
       since: Get data only changed since this timestamp (inclusive) or None
         for all data.
-        
+      location: Currently a string containing our search base, later we
+        may support hostname and additional parameters.
+
     Returns:
       instance of AutomountMap
     """
@@ -313,7 +362,7 @@ class LdapSource(base.Source):
     search_filter = '(&(objectclass=automountMap)(ou=auto.master))'
     self.log.debug('retrieving automount master map.')
     self.Search(search_base=search_base, search_filter=search_filter,
-                  search_scope=search_scope, attrs=['dn'])
+                search_scope=search_scope, attrs=['dn'])
 
     search_base = None
     for obj in self:
@@ -325,7 +374,7 @@ class LdapSource(base.Source):
       raise error.EmptyMap
 
     self.log.debug('found ou=auto.master at %s', search_base)
-    master_map =  self.GetAutomountMap(location=search_base)
+    master_map = self.GetAutomountMap(location=search_base)
 
     # fix our location attribute to contain the data we
     # expect returned to us later, namely the new search base(s)
@@ -336,14 +385,14 @@ class LdapSource(base.Source):
       # and strip the space seperated options
       map_entry.location = map_entry.location.split(' ')[0]
       self.log.debug('master map has: %s' % map_entry.location)
-                     
+
     return master_map
 
   def Verify(self, since=None):
     """Verify that this source is contactable and can be queried for data."""
     if since is None:
       # one minute in the future
-      since = time.time() + 60
+      since = time.gmtime(time.time() + 60)
     results = self.GetPasswdMap(since=since)
     return len(results)
 
@@ -358,21 +407,22 @@ class UpdateGetter(object):
       ldap_ts_string: An LDAP timestamp string in the format %Y%m%d%H%M%SZ
 
     Returns:
-      number of seconds since epoch.
+      a time.struct_time
     """
     t = time.strptime(ldap_ts_string, '%Y%m%d%H%M%SZ')
-    return calendar.timegm(t)
+    return t
 
   def FromTimestampToLdap(self, ts):
     """Transforms nss_cache internal timestamp into a LDAP timestamp.
 
     Args:
-      ts: number of seconds since epoch
+      ts: a time.struct_time
+
     Returns:
       LDAP format timestamp string.
     """
-    ts = time.gmtime(ts)
-    return time.strftime('%Y%m%d%H%M%SZ', ts)
+    t = time.strftime('%Y%m%d%H%M%SZ', ts)
+    return t
 
   def GetUpdates(self, source, search_base, search_filter,
                  search_scope, since):
@@ -399,7 +449,7 @@ class UpdateGetter(object):
       # since openldap disallows modifyTimestamp "greater than" we have to
       # increment by one second.
       ts = int(ts.rstrip('Z')) + 1
-      ts = '%sZ' % ts      
+      ts = '%sZ' % ts
       search_filter = ('(&%s(modifyTimestamp>=%s))' % (search_filter, ts))
 
     if search_scope == 'base':
@@ -450,22 +500,24 @@ class PasswdUpdateGetter(UpdateGetter):
   def __init__(self):
     super(PasswdUpdateGetter, self).__init__()
     self.attrs = ['uid', 'uidNumber', 'gidNumber', 'gecos', 'cn',
-                  'homeDirectory', 'loginShell']
+                  'homeDirectory', 'loginShell', 'fullName']
     self.essential_fields = ['uid', 'uidNumber', 'gidNumber', 'homeDirectory']
 
   def CreateMap(self):
     """Returns a new PasswdMap instance to have PasswdMapEntries added to it."""
-    return maps.PasswdMap()
+    return passwd.PasswdMap()
 
   def Transform(self, obj):
     """Transforms a LDAP posixAccount data structure into a PasswdMapEntry."""
 
-    pw = maps.PasswdMapEntry()
+    pw = passwd.PasswdMapEntry()
 
     if 'gecos' in obj:
       pw.gecos = obj['gecos'][0]
     elif 'cn' in obj:
       pw.gecos = obj['cn'][0]
+    elif 'fullName' in obj:
+      pw.gecos = obj['fullName'][0]
     else:
       raise ValueError('Neither gecos nor cn found')
 
@@ -495,12 +547,12 @@ class GroupUpdateGetter(UpdateGetter):
 
   def CreateMap(self):
     """Return a GroupMap instance."""
-    return maps.GroupMap()
+    return group.GroupMap()
 
   def Transform(self, obj):
     """Transforms a LDAP posixGroup object into a group(5) entry."""
 
-    gr = maps.GroupMapEntry()
+    gr = group.GroupMapEntry()
 
     gr.name = obj['cn'][0]
     # group passwords are deferred to gshadow
@@ -528,11 +580,11 @@ class ShadowUpdateGetter(UpdateGetter):
 
   def CreateMap(self):
     """Return a ShadowMap instance."""
-    return maps.ShadowMap()
+    return shadow.ShadowMap()
 
   def Transform(self, obj):
     """Transforms an LDAP shadowAccont object into a shadow(5) entry."""
-    shadow_ent = maps.ShadowMapEntry()
+    shadow_ent = shadow.ShadowMapEntry()
     shadow_ent.name = obj['uid'][0]
     # TODO(jaq): does nss_ldap check the contents of the userPassword
     # attribute?
@@ -555,7 +607,7 @@ class ShadowUpdateGetter(UpdateGetter):
       shadow_ent.flag = 0
     if 'userPassword' in obj:
       passwd = obj['userPassword'][0]
-      if passwd[:7] == "{CRYPT}":
+      if passwd[:7].lower() == '{crypt}':
         shadow_ent.passwd = passwd[7:]
       else:
         logging.info('Ignored password that was not in crypt format')
@@ -572,21 +624,21 @@ class NetgroupUpdateGetter(UpdateGetter):
 
   def CreateMap(self):
     """Return a NetgroupMap instance."""
-    return maps.NetgroupMap()
+    return netgroup.NetgroupMap()
 
   def Transform(self, obj):
     """Transforms an LDAP nisNetgroup object into a netgroup(5) entry."""
-    netgroup_ent = maps.NetgroupMapEntry()
+    netgroup_ent = netgroup.NetgroupMapEntry()
     netgroup_ent.name = obj['cn'][0]
 
-    entries = []
+    entries = set()
     if 'memberNisNetgroup' in obj:
-      entries.extend(obj['memberNisNetgroup'])
+      entries.update(obj['memberNisNetgroup'])
     if 'nisNetgroupTriple' in obj:
-      entries.extend(obj['nisNetgroupTriple'])  
+      entries.update(obj['nisNetgroupTriple'])
 
     # final data is stored as a string in the object
-    netgroup_ent.entries = ' '.join(entries)
+    netgroup_ent.entries = ' '.join(sorted(entries))
 
     return netgroup_ent
 
@@ -601,11 +653,11 @@ class AutomountUpdateGetter(UpdateGetter):
 
   def CreateMap(self):
     """Return a AutomountMap instance."""
-    return maps.AutomountMap()
+    return automount.AutomountMap()
 
   def Transform(self, obj):
     """Transforms an LDAP automount object into an autofs(5) entry."""
-    automount_ent = maps.AutomountMapEntry()
+    automount_ent = automount.AutomountMapEntry()
     automount_ent.key = obj['cn'][0]
 
     automount_information = obj['automountInformation'][0]
@@ -620,6 +672,3 @@ class AutomountUpdateGetter(UpdateGetter):
 
     return automount_ent
 
-
-# Finally, register the Source
-base.RegisterImplementation(LdapSource)
